==> rheolef_6.5.orig.tar.gz: rheolef-6.5/nfem/geo_element/tensor4.h <==
# ifndef _RHEOLEF_TENSOR4_H
# define _RHEOLEF_TENSOR4_H
///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/// 
/// =========================================================================

/*Class:tensor4
NAME: @code{tensor4} - a fouth order tensor
@cindex tensor4
@clindex tensor4
SYNOPSYS:
@noindent
The @code{tensor4} class defines a fourth tensor where
indices varie from zero to 2 (aka 3D physical space).
End:
*/

#include "rheolef/point.h"
#include "rheolef/tensor.h"
namespace rheolef {

//<tensor4:
template<class T>
class tensor4_basic {
public:

  typedef size_t size_type;
  typedef T      element_type;
  typedef T      float_type;

// allocators:

  tensor4_basic (const T& init_val = 0);
  tensor4_basic (const tensor4_basic<T>& a);

#ifdef _RHEOLEF_HAVE_STD_INITIALIZER_LIST
        tensor4_basic (const std::initializer_list<std::initializer_list<
			     std::initializer_list<std::initializer_list<T> > > >& il);
#endif // _RHEOLEF_HAVE_STD_INITIALIZER_LIST

// affectation:

  tensor4_basic<T>& operator= (const tensor4_basic<T>& a);
  tensor4_basic<T>& operator= (const T& val);

// accessors:

  T&       operator()(size_type i, size_type j, size_type k, size_type l);
  const T& operator()(size_type i, size_type j, size_type k, size_type l) const;

// algebra:

  tensor4_basic<T>& operator*= (const T& k);
  tensor4_basic<T>& operator/= (const T& k) { return operator*= (1./k); }
  tensor4_basic<T>  operator+ (const tensor4_basic<T>& b) const;
  tensor4_basic<T>  operator- (const tensor4_basic<T>& b) const;

// io:
  std::ostream& put (std::ostream& out, size_type d=3) const;

// data:
protected:
  T _x [3][3][3][3];
};
typedef tensor4_basic<Float> tensor4;

// nonlinear algebra:
template <class T>
T norm2 (const tensor4_basic<T>&);
template <class T>
T norm (const tensor4_basic<T>& a) { return sqrt(norm2(a)); }
template <class T>
tensor4_basic<T> dexp (const tensor_basic<T>& a, size_t d = 3);

// algebra:
template <class T>
tensor_basic<T> ddot (const tensor4_basic<T>&, const tensor_basic<T>&);
template <class T>
tensor_basic<T> ddot (const tensor_basic<T>&, const tensor4_basic<T>&);
//>tensor4:
// -----------------------------------------------------------------------
// inlined
// -----------------------------------------------------------------------
template<class T> struct  float_traits<tensor4_basic<T> > { typedef typename float_traits<T>::type type; };
template<class T> struct scalar_traits<tensor4_basic<T> > { typedef T type; };

template<class T>
inline
tensor4_basic<T>::tensor4_basic (const T& init_val)
{ 
    operator= (init_val);
}
template<class T>
inline
tensor4_basic<T>::tensor4_basic (const tensor4_basic<T>& a)
{
    operator= (a);
}
template<class T>
inline
T&
tensor4_basic<T>::operator() (size_type i, size_type j, size_type k, size_type l)  
{
    return _x[i%3][j%3][k%3][l%3];
}
template<class T>
inline
const T&
tensor4_basic<T>::operator() (size_type i, size_type j, size_type k, size_type l) const
{
    return _x[i%3][j%3][k%3][l%3];
}
template <class T>
inline
tensor4_basic<T>
operator* (const T& k, const tensor4_basic<T>& a)
{
  tensor4_basic<T> b = a;
  b *= k;
  return b;
}
template <class T>
inline
tensor4_basic<T>
operator* (const tensor4_basic<T>& a, const T& k)
{
  return k*a;
}
template <class T>
inline
tensor4_basic<T>
operator/ (const tensor4_basic<T>& a, const T& k)
{
  return (1./k)/a;
}

}// namespace rheolef
# endif /* _RHEOLEF_TENSOR4_H */

==> rheolef_6.6.orig.tar.gz: rheolef-6.6/nfem/geo_element/tensor4.h <==
# ifndef _RHEOLEF_TENSOR4_H
# define _RHEOLEF_TENSOR4_H
///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/// 
/// =========================================================================

/*Class:tensor4
NAME: @code{tensor4} - a fouth order tensor
@cindex tensor4
@clindex tensor4
SYNOPSYS:
@noindent
The @code{tensor4} class defines a fourth tensor where
indices varie from zero to 2 (aka 3D physical space).
End:
*/

#include "rheolef/point.h"
#include "rheolef/tensor.h"
namespace rheolef {

//<tensor4:
template<class T>
class tensor4_basic {
public:

  typedef size_t size_type;
  typedef T      element_type;
  typedef T      float_type;

// allocators:

  tensor4_basic ();
  explicit tensor4_basic (const T& init_val);
  tensor4_basic (const tensor4_basic<T>& a);
  static tensor4_basic<T> eye (size_type d = 3);

#ifdef _RHEOLEF_HAVE_STD_INITIALIZER_LIST
        tensor4_basic (const std::initializer_list<std::initializer_list<
			     std::initializer_list<std::initializer_list<T> > > >& il);
#endif // _RHEOLEF_HAVE_STD_INITIALIZER_LIST

// affectation:

  tensor4_basic<T>& operator= (const tensor4_basic<T>& a);
  tensor4_basic<T>& operator= (const T& val);

// accessors:

  T&       operator()(size_type i, size_type j, size_type k, size_type l);
  const T& operator()(size_type i, size_type j, size_type k, size_type l) const;

  tensor_basic<T>&  operator()(size_type i, size_type j);
  const tensor_basic<T>& operator()(size_type i, size_type j) const;

// algebra:

  tensor4_basic<T>& operator*= (const T& k);
  tensor4_basic<T>& operator/= (const T& k) { return operator*= (1./k); }
  tensor4_basic<T>  operator+ (const tensor4_basic<T>& b) const;
  tensor4_basic<T>  operator- (const tensor4_basic<T>& b) const;

// io:
  std::ostream& put (std::ostream& out, size_type d=3) const;

// data:
protected:
  tensor_basic<tensor_basic<T> > _x;
};
typedef tensor4_basic<Float> tensor4;

// nonlinear algebra:
template <class T>
T norm2 (const tensor4_basic<T>&);
template <class T>
T norm (const tensor4_basic<T>& a) { return sqrt(norm2(a)); }
template <class T>
tensor4_basic<T> dexp (const tensor_basic<T>& a, size_t d = 3);

// algebra:
template <class T>
tensor_basic<T> ddot (const tensor4_basic<T>&, const tensor_basic<T>&);
template <class T>
tensor_basic<T> ddot (const tensor_basic<T>&, const tensor4_basic<T>&);
//>tensor4:
// -----------------------------------------------------------------------
// inlined
// -----------------------------------------------------------------------
template<class T> struct  float_traits<tensor4_basic<T> > { typedef typename float_traits<T>::type type; };
template<class T> struct scalar_traits<tensor4_basic<T> > { typedef T type; };

template<class T>
inline
tensor4_basic<T>::tensor4_basic()
 : _x (tensor_basic<T>(T()))
{ 
}
template<class T>
inline
tensor4_basic<T>::tensor4_basic(const T& init_val)
 : _x (tensor_basic<T>(init_val))
{ 
}
template<class T>
inline
tensor4_basic<T>::tensor4_basic (const tensor4_basic<T>& a)
 : _x (tensor_basic<T>(T()))
{
    operator= (a);
}
template<class T>
inline
tensor_basic<T>&
tensor4_basic<T>::operator() (size_type i, size_type j)  
{
    return _x(i,j);
}
template<class T>
inline
const tensor_basic<T>&
tensor4_basic<T>::operator() (size_type i, size_type j) const
{
    return _x(i,j);
}
template<class T>
inline
T&
tensor4_basic<T>::operator() (size_type i, size_type j, size_type k, size_type l)  
{
    return _x(i,j)(k,l);
}
template<class T>
inline
const T&
tensor4_basic<T>::operator() (size_type i, size_type j, size_type k, size_type l) const
{
    return _x(i,j)(k,l);
}
template <class T>
inline
tensor4_basic<T>
operator* (const T& k, const tensor4_basic<T>& a)
{
  tensor4_basic<T> b = a;
  b *= k;
  return b;
}
template <class T>
inline
tensor4_basic<T>
operator* (const tensor4_basic<T>& a, const T& k)
{
  return k*a;
}
template <class T>
inline
tensor4_basic<T>
operator/ (const tensor4_basic<T>& a, const T& k)
{
  return (1./k)/a;
}
// inputs/outputs:
#ifdef TODO
template<class T>
inline
std::istream& operator>> (std::istream& in, tensor4_basic<T>& a)
{
    return a.get (in);
}
#endif // TODO
template<class T>
inline
std::ostream& operator<< (std::ostream& out, const tensor4_basic<T>& a)
{
    return a.put (out);
}

}// namespace rheolef
# endif /* _RHEOLEF_TENSOR4_H */
